
# q-expansion of newform 1859.2.a.i, downloaded from the LMFDB on 17 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1859
weight = 2
poly_data = [1, -1, -4, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-1, -4, 0, 1], 1], [[-1, 3, 1, -1], 1]]

hecke_ring_character_values = None
aps_data = [[-1, -1, -1, 0], [0, 0, -1, -1], [0, 2, 0, 0], [-1, 0, 1, -1], [-1, 0, 0, 0], [0, 0, 0, 0], [2, -2, 2, 0], [-2, 2, 3, 3], [-1, 2, 1, 1], [-2, 0, 0, -2], [0, -4, -2, -4], [-4, 0, -4, 0], [-3, -4, -5, -1], [6, -2, -2, 0], [4, 2, 0, 2], [-2, -2, -3, -1], [4, -4, -2, -2], [-2, 2, 2, -2], [-2, 2, -2, 4], [4, 0, 6, 4], [-4, 0, 3, -3], [-4, -8, -4, 2], [1, 0, 3, 1], [-2, 2, 4, 2], [-6, -4, 4, 6], [-4, -4, 2, 0], [-3, -2, -1, 3], [0, 4, 4, 6], [-4, -4, -7, 3], [-6, -2, -7, -1], [0, 4, 8, 6], [-4, 2, -4, -4], [2, -2, 0, -4], [-8, -2, -6, -2], [-1, 4, 5, -3], [0, 4, 0, 0], [-3, -2, 3, -7], [-2, 0, -10, 0], [2, 2, 3, -5], [-4, 6, 8, 6], [4, 8, 8, 4], [-14, -6, -7, -5], [0, -4, -9, 3], [-13, -4, -3, 5], [0, 8, 13, 11], [-8, 4, 1, 1], [0, 0, 0, 6], [-6, -6, -2, -6], [-18, -2, 3, 7], [2, -8, -8, -4], [10, 4, 2, 6], [-2, -10, -11, 5], [-9, 4, 1, 1], [7, 2, -7, -3], [7, 6, -3, -9], [8, -10, -10, -12], [-17, 6, 5, 7], [-10, -6, -5, -9], [2, 10, 0, 8], [4, 8, 7, 5], [-6, -4, -16, -4], [6, -12, 0, 0], [4, -4, 4, 4], [-8, 4, 5, -3], [3, -6, 5, -1], [-8, 8, 6, 4], [2, 10, -2, 0], [6, -4, -6, 0], [12, -6, 2, 0], [9, 0, 3, -9], [-4, -4, -10, 2], [1, 8, 3, 1], [16, 4, -3, -11], [0, 10, 0, -6], [-2, 12, -6, 2], [0, -2, 8, -2], [14, 6, 19, 1], [-12, -14, -10, 0], [12, -16, -6, -8], [-2, -4, 4, -4], [-17, -6, -7, -3], [10, 6, 2, -12], [-13, 0, -3, -1], [2, -14, 5, 3], [6, -20, -8, -4], [-13, -14, -15, -7], [-10, 0, 10, 0], [-8, -4, 3, -3], [12, 12, -1, -11], [2, 10, 18, 4], [0, -4, -8, 4], [4, 8, 12, 8], [-10, 2, 6, 6], [14, 16, 26, 12], [8, 6, 16, 12], [10, 2, 10, -8], [12, -10, -6, 2], [1, -6, 11, -7], [16, 10, 18, 0], [-7, -4, -5, -5], [-2, -6, 10, 4], [3, 8, -3, 9], [18, 12, 2, -6], [-22, 4, -2, 8], [8, 10, 6, -2], [-4, 4, 4, -10], [18, -12, -2, -6], [7, -16, -15, -7], [0, -4, -13, -1], [-12, -2, 4, -14], [6, 16, 22, 10], [-12, 12, -5, 1], [2, -12, -16, -6], [-6, 14, -8, 0], [18, 14, 20, 4], [-11, 6, -5, 9], [-6, -4, -22, -10], [-20, 0, 5, -3], [2, 8, 0, 12], [16, 6, 12, 0], [-4, 24, 4, 2], [32, -16, -6, -4], [8, 0, -8, 16], [4, -2, 8, -4], [-8, 18, 6, -6], [8, -2, -14, 6], [-16, -8, -2, 4], [12, -12, -4, 4], [-19, -2, 7, 11], [17, 8, 11, -1], [21, -12, -9, 5], [-8, -8, -4, -8], [-4, -4, 7, -1], [3, 26, 5, -5], [3, 0, -7, 1], [7, 8, 5, -3], [2, 18, -2, -8], [5, 8, -9, 5], [-6, 0, 0, 4], [-6, -6, -8, 8], [9, -16, -13, -11], [6, -8, 8, 12], [-21, -6, -11, -7], [-10, 10, 5, -3], [-15, 10, -9, 9], [10, -16, -8, -12], [-16, -4, 7, 5], [-10, 14, 10, 4], [-28, 0, -5, -5], [-6, 16, 14, 20], [-16, 36, 13, 7], [30, 6, 1, -9], [-16, -12, -25, -1], [-18, -10, -12, -6], [11, -2, 9, -3], [-5, -14, 17, -3], [-14, -16, -12, -4], [20, -4, -10, 8], [20, -18, -8, -4], [-4, 0, -13, -15], [-10, -8, -8, -22], [-14, 26, 0, 2], [18, 6, 7, 15], [16, -8, -3, -15], [12, -16, 8, 0], [14, 2, -8, 0], [12, 16, 11, 7], [-6, 6, 22, 6], [-11, -4, 3, 23], [14, -8, -12, 0], [-1, -24, -11, -21], [-36, 2, 2, -2], [0, -2, 2, -18], [-14, 24, 12, 0], [-17, 26, 9, 1], [7, -2, 5, 19], [15, 0, -11, -5], [-8, 26, 14, 12], [-22, -2, -2, -6], [-14, 16, 20, 2], [24, 18, 4, -14], [-14, 8, -22, -4], [10, 6, 7, 9], [14, 0, 8, -12], [-24, -10, -6, 8], [25, -18, -9, 1], [16, 22, -4, -6], [32, 14, 6, -2], [-20, -12, -15, -21], [-14, 26, 11, 15], [1, 2, 15, 25], [3, -28, -23, -13], [24, 8, 5, -3], [16, -12, -8, -12], [14, 8, -4, 14], [18, 18, 11, 25], [4, -24, -1, -11], [10, 6, 9, 3], [18, 0, 4, -16], [-4, 22, -4, 8], [-22, -4, -8, -12], [-8, 14, -10, -8], [0, -2, -6, -2], [-8, 14, 0, 8], [8, 6, -14, -12], [-38, 22, 15, 1], [18, 0, 18, -6], [18, -10, -4, -12], [-12, -4, -9, 17], [-12, 0, 20, 0], [-12, -26, -22, 8], [22, 20, -8, -4], [0, -4, -8, -8], [32, 2, -4, 6], [13, -24, -13, 5], [-20, -4, -20, -2], [20, -20, -2, -6], [-22, -22, -16, 8], [-18, 38, 24, 20], [-2, -14, -4, 18], [-12, -12, 14, 0], [12, -32, -20, -24], [20, 30, 30, -4], [14, 24, 32, 14], [-26, 10, 14, 0], [10, 12, 4, 10], [-1, -22, -15, -29], [-4, 32, 16, 12], [-8, 2, 26, 6], [26, 22, 31, 3], [22, 14, -1, -15], [24, 14, 18, -8], [2, -14, -7, -19], [18, -10, -8, -16], [40, -4, -12, -4], [-2, 30, 19, 7], [-2, 18, 12, -2], [-16, -8, -15, 3], [-16, -20, 1, 13], [-35, -14, -13, -1], [-22, -14, -20, -6], [14, 28, 6, -2], [4, 8, -4, -8], [-32, -22, -14, 8], [-4, 12, 25, 11], [-20, -18, -12, -2], [-32, -10, -14, -8], [36, 12, 32, 12], [-42, -6, -9, 3], [0, 4, -20, -22], [-22, 0, -8, -16], [0, 20, -13, -7], [8, -16, -22, 14], [-3, 10, 11, -3], [-16, 6, 28, 16], [-26, 6, -5, -5], [-11, -10, 3, -31], [-1, -8, 5, 19], [19, -10, -3, 3], [11, -16, -11, -23], [-14, -14, -6, -14], [8, -4, -12, 28], [21, 16, -1, 7], [-28, -12, 16, 16], [-42, -6, -20, -4], [-4, -8, -3, 17], [6, -14, -6, 10], [24, -16, 2, 6], [-8, -16, -12, -16], [4, 6, 14, 8], [-51, 2, -1, 1], [2, 8, -16, -10], [-12, 18, 18, -14], [30, 8, -20, -10], [-10, 2, -16, -6], [-6, -8, 6, 6], [30, 18, -20, -22], [-7, -8, 19, 27], [3, 24, 17, 41], [28, 10, -8, 4], [52, -6, 12, 0], [14, -12, 20, -10], [-26, -24, -6, 8], [-17, 28, 33, 25], [17, 16, 19, 21], [32, -4, -9, 19], [38, -6, -1, 9], [-11, -36, -33, -5], [-22, -22, -10, 6], [-22, 28, 18, 20], [-25, 22, -23, 3], [36, 28, 32, 12], [8, -12, -23, -41], [2, -42, -19, -15], [10, -10, 2, 10], [34, -36, -14, -8], [37, 14, 27, -1], [-10, -26, 17, 3], [-22, -10, 6, -26], [2, 26, 17, 1], [12, -30, -10, -26], [-23, -20, -9, -13], [0, 0, 8, 0], [-12, -28, -1, -13], [35, -34, -7, -3], [-10, -2, -32, -18], [-15, -2, -9, -5], [3, -18, -3, -21], [0, -18, -10, -14], [2, 8, -14, 12], [-34, -4, -4, -10], [-8, -24, -16, -30], [-40, -18, 2, 0], [-15, -6, 3, 31], [-51, 0, 7, -7], [-42, 2, -8, -4], [-28, -24, -2, 24], [-33, 32, 17, 19], [4, 2, 30, 4], [24, 6, 26, -10], [0, -18, -16, -36], [24, 18, 28, 18], [-12, -4, -10, 10], [18, 14, 24, -14], [14, 4, 32, -4], [-15, -6, -1, 17], [-10, 4, -24, -24], [-2, -20, -18, -20], [-38, -16, -24, -18], [4, 38, 28, 28], [-18, 16, 32, 24], [-2, 18, 5, -25], [4, 14, 10, 6], [33, 16, -1, -15], [-35, 16, -1, -3], [20, -14, 16, 8], [-12, 2, -22, -14], [2, 32, -8, -8], [8, 22, 22, 12], [30, 40, 24, -16], [14, 2, -2, -16], [-10, -12, 4, -20], [-56, 4, 5, 3], [12, 4, 32, -12], [0, 32, -14, 6], [14, 26, 22, 8], [-10, -38, 7, 1], [8, -28, -8, -4], [-14, -18, -35, -15], [-10, -46, -47, 3], [-9, -12, -7, 27], [-64, -12, -22, 0], [-36, 12, 16, 26], [-26, -14, -2, 22], [8, 8, 30, 6], [-16, -2, 4, -16], [4, 48, 20, -10], [-28, 4, -12, 8], [38, 10, 6, -14], [28, -42, -28, -20], [-53, -28, -27, 5], [14, 8, 16, 18], [28, -32, -8, -20], [-11, 20, -13, -7], [-31, -2, -25, 7], [54, 10, 13, -13], [25, -8, 3, 11], [38, 14, 3, 15], [-12, -4, 10, 10], [-9, -22, -31, -27], [40, 16, 9, 5], [-53, 16, -3, 3], [0, 4, -12, 12], [4, 0, -30, -12], [42, -14, -13, 1], [24, 36, 10, -8], [36, 24, 8, 8], [-34, 0, 16, -24], [2, 18, 34, 28], [-12, 0, 2, -8], [-38, -16, -2, -12], [-24, 10, -6, -20], [-4, -20, 0, 8], [21, 40, 27, 3], [18, 14, -29, -25], [-24, 16, 28, 36], [18, 18, -8, 38], [22, -22, 29, -1], [-12, -36, 20, 12], [21, 22, 23, -21], [-32, 6, -22, -2], [40, -22, -14, 0], [4, 42, 42, -6], [-19, -2, 35, 15], [-10, 10, -36, 0], [-2, 40, 4, 6], [-10, -22, 3, 27], [-38, 0, 4, 12], [-4, 12, 8, 20], [-5, 0, 17, 25], [-29, -12, -15, -3], [39, -30, -11, -17], [2, -38, -37, -5], [2, 6, 12, -10], [0, 28, 23, 21], [-32, -30, -22, -22], [24, 8, -24, 2], [7, 44, 5, -7], [-14, 38, 20, -12], [22, 22, 22, -6], [-8, -20, -46, 6], [-2, 18, -6, -6], [-20, 24, 13, 17], [-22, 58, 40, 18], [-6, -2, -32, -32], [-9, 2, -27, -1], [26, -14, -7, -3], [12, 4, -28, -16], [-50, -26, 6, 22], [-10, 26, -2, 18], [-5, 46, 21, 3], [-3, 0, -5, 3], [66, 2, 4, -8], [12, 10, 12, 0], [-6, 16, 6, 2], [-24, -2, 2, -16], [-39, 10, 23, 11], [-18, -24, -40, -16], [8, -8, 6, -2], [-26, -26, -32, -42]]
